@@ -16,6 +16,7 @@ import sys
 import time
 import webbrowser
 import yaml
+import copy
 
 import nf_core.list, nf_core.utils
 
@@ -79,13 +80,14 @@ class PipelineSchema(object):
             self.load_schema()
             num_params = self.validate_schema()
             self.get_schema_defaults()
-            log.info("[green]\[✓] Pipeline schema looks valid[/] [dim](found {} params)".format(num_params))
+            self.validate_default_params()
+            log.info("[green][✓] Pipeline schema looks valid[/] [dim](found {} params)".format(num_params))
         except json.decoder.JSONDecodeError as e:
             error_msg = "[bold red]Could not parse schema JSON:[/] {}".format(e)
             log.error(error_msg)
             raise AssertionError(error_msg)
         except AssertionError as e:
-            error_msg = "[red]\[✗] Pipeline schema does not follow nf-core specs:\n {}".format(e)
+            error_msg = "[red][✗] Pipeline schema does not follow nf-core specs:\n {}".format(e)
             log.error(error_msg)
             raise AssertionError(error_msg)
 
@@ -159,14 +161,35 @@ class PipelineSchema(object):
             assert self.schema is not None
             jsonschema.validate(self.input_params, self.schema)
         except AssertionError:
-            log.error("[red]\[✗] Pipeline schema not found")
+            log.error("[red][✗] Pipeline schema not found")
             return False
         except jsonschema.exceptions.ValidationError as e:
-            log.error("[red]\[✗] Input parameters are invalid: {}".format(e.message))
+            log.error("[red][✗] Input parameters are invalid: {}".format(e.message))
             return False
-        log.info("[green]\[✓] Input parameters look valid")
+        log.info("[green][✓] Input parameters look valid")
         return True
 
+    def validate_default_params(self):
+        """
+        Check that all default parameters in the schema are valid
+        Ignores 'required' flag, as required parameters might have no defaults
+        """
+        try:
+            assert self.schema is not None
+            # Make copy of schema and remove required flags
+            schema_no_required = copy.deepcopy(self.schema)
+            if "required" in schema_no_required:
+                schema_no_required.pop("required")
+            for group_key, group in schema_no_required["definitions"].items():
+                if "required" in group:
+                    schema_no_required["definitions"][group_key].pop("required")
+            jsonschema.validate(self.schema_defaults, schema_no_required)
+        except AssertionError:
+            log.error("[red][✗] Pipeline schema not found")
+        except jsonschema.exceptions.ValidationError as e:
+            raise AssertionError("Default parameters are invalid: {}".format(e.message))
+        log.info("[green][✓] Default parameters look valid")
+
     def validate_schema(self, schema=None):
         """
         Check that the Schema is valid
@@ -264,19 +287,15 @@ class PipelineSchema(object):
         """ Make a new pipeline schema from the template """
         self.schema_from_scratch = True
         # Use Jinja to render the template schema file to a variable
-        # Bit confusing sorry, but cookiecutter only works with directories etc so this saves a bunch of code
-        templateLoader = jinja2.FileSystemLoader(
-            searchpath=os.path.join(
-                os.path.dirname(os.path.realpath(__file__)), "pipeline-template", "{{cookiecutter.name_noslash}}"
-            )
+        env = jinja2.Environment(
+            loader=jinja2.PackageLoader("nf_core", "pipeline-template"), keep_trailing_newline=True
         )
-        templateEnv = jinja2.Environment(loader=templateLoader)
-        schema_template = templateEnv.get_template("nextflow_schema.json")
-        cookiecutter_vars = {
+        schema_template = env.get_template("nextflow_schema.json")
+        template_vars = {
             "name": self.pipeline_manifest.get("name", os.path.dirname(self.schema_filename)).strip("'"),
             "description": self.pipeline_manifest.get("description", "").strip("'"),
         }
-        self.schema = json.loads(schema_template.render(cookiecutter=cookiecutter_vars))
+        self.schema = json.loads(schema_template.render(template_vars))
         self.get_schema_defaults()
 
     def build_schema(self, pipeline_dir, no_prompts, web_only, url):
@@ -424,7 +443,7 @@ class PipelineSchema(object):
             if self.no_prompts or self.schema_from_scratch:
                 return True
             if Confirm.ask(
-                ":question: Unrecognised [white bold]'params.{}'[/] found in schema but not pipeline! [yellow]Remove it?".format(
+                ":question: Unrecognised [white bold]'params.{}'[/] found in the schema but not in the pipeline config! [yellow]Remove it?".format(
                     p_key
                 )
             ):
@@ -436,14 +455,16 @@ class PipelineSchema(object):
         Add anything that's found in the Nextflow params that's missing in the pipeline schema
         """
         params_added = []
+        params_ignore = self.pipeline_params.get("schema_ignore_params", "").strip("\"'").split(",")
+        params_ignore.append("schema_ignore_params")
         for p_key, p_val in self.pipeline_params.items():
             # Check if key is in schema parameters
-            if not p_key in self.schema_params:
+            if p_key not in self.schema_params and p_key not in params_ignore:
                 if (
                     self.no_prompts
                     or self.schema_from_scratch
                     or Confirm.ask(
-                        ":sparkles: Found [white bold]'params.{}'[/] in pipeline but not in schema. [blue]Add to pipeline schema?".format(
+                        ":sparkles: Found [white bold]'params.{}'[/] in the pipeline config, but not in the schema. [blue]Add to pipeline schema?".format(
                             p_key
                         )
                     )
